#if !defined(M_POLFIT_H)
#define M_POLFIT_H

/*
   $Log: polfit.h,v $
   Revision 1.1.2.1  2005/03/15 14:26:06  kraemer
   *** empty log message ***

*/


struct SPOLFITPAR {
   unsigned long ulFlag;              /* POLFIT_NOERR,... */
   int np;                            /* inp: # of parameters */
   int nDF;                           /* out: # degrees of freedom */
   double dChisq,                     /* out: chisquare */
          dProb,                      /* out: probability */
          dConf;                      /* inp: (0..1) confidence limit */
   double *pdPar, *pdErr;             /* out: [np] parameters, errors */
   double *pdCovar;                   /* out: [np*np] covariance matrix */
   long   *plCol, *plRow;
};

int polfitParAlloc  ( int, struct SPOLFITPAR *, unsigned long );
int polfitParDup    (      struct SPOLFITPAR *, struct SPOLFITPAR *, unsigned long );
int polfitParFree   (      struct SPOLFITPAR *, unsigned long );
int polfitParInit   (      struct SPOLFITPAR *, unsigned long );
int polfitParInspect(      struct SPOLFITPAR *, unsigned long );
int polfitParIsValid(      struct SPOLFITPAR *, unsigned long );

struct SPOLFITPAR *_polfitParTest( unsigned long );

/*
   --- structure describing data
*/
struct SPOLFITDATA {
   unsigned long lFlag;
   int nn;                            /* inp: # of data points */
   double *pdW;                       /* scratch: weights */
   double *pdX, *pddXm, *pddXp;       /* inp: x-values and -errors */
   double *pdY, *pddYm, *pddYp;       /* inp: y-values and -errors */
   double *pdF, *pddFm, *pddFp;       /* out: function values and -errors */
};

int polfitDataAlloc  ( int, struct SPOLFITDATA *, unsigned long );
int polfitDataFree   (      struct SPOLFITDATA *, unsigned long );
int polfitDataDup    (      struct SPOLFITDATA *, struct SPOLFITDATA *, unsigned long );
int polfitDataInspect(      struct SPOLFITDATA *, unsigned long );

int polfitFit( struct SPOLFITDATA *, struct SPOLFITPAR *, unsigned long );
int polfitFct( struct SPOLFITDATA *, struct SPOLFITPAR *, unsigned long );

                                      /* flags */
#define POLFIT_DEBUG 0x80000000       /* global debug */
#define POLFIT_NOERR 0x00010000       /* unknown data errors on input */

                                      /* error handling */
#define POLFIT_ERROR     0x80000000
#define POLFIT_WARNING   0x40000000
#define POLFIT_ERRORMASK 0xF0000000

char *polfitStrError( int );

enum {
   ERRPOLFIT_SYS=1,
   ERRPOLFIT_BADADDR,
   ERRPOLFIT_BADPARCOUNT,
   ERRPOLFIT_BADDATACOUNT,
   ERRPOLFIT_NDF,
   ERRPOLFIT_INVMAT,
};

#endif

