#ifndef TRXTARG_H
/*
   Various definitions for target materials

   Copyright  1992-2009 M.Kraemer, GSI, D-64291 Darmstadt

   $Log: trxtarg.h,v $
   Revision 1.4  2008/06/20 14:36:15  kraemer
   x98-libs

   Revision 1.3  2002/11/25 16:37:36  kraemer
   TEA (tri-ethyl-amine) added

   Revision 1.2  2002/01/22 17:31:29  kraemer
   strict for Aix, OS-dependent makefile, bugfix in mdose evaluation

   Revision 1.1  2001/01/09 14:54:42  kraemer
   *** empty log message ***



*/
#define TRXTARG_H

#include "ulists.h"
#include "m.h"
#include "cmderr.h"
#include "trxproj.h"

                                      /* */
                                      /* global flags */
#define TRXTARG_DEBUG  0x80000000     /* general check flag */
#define TRXTARG_QUIET  0x10000000     /* suppress error messages */
#define TRXTARG_NOMSG  TRXTARG_QUIET
/*
   --- 
*/
                                      /* */
struct STRXPARENT {
   double dFraction;
   char cAtom[4];                     /* referred atom name */
   int iAtom;                         /* referred atom number (0..n-1)*/
   int iSh;                           /* referred atomic shell number (1..n) */ 
};                                          
                                      /* structure describing an electron shell */
struct STRXSHELL {
   int in, il;                        /* quantum numbers */ 
   int    ne;                         /* # of electrons */
   double dEBind,                     /* experimental binding energy */
          dEKinetic;                  /* mean kinetic energy a la Slater */
   double ds;                         /* shielding constant */
   int nParent;
   struct STRXPARENT *psParent;       /* parentage */  
};

                                      /* structure describing an atom */
struct STRXATOM {
   int nN;                            /* # of atoms of following species */
   char *pcName;                      /* element name */ 
   struct STRXZA sZA;                 /* atom species */
                                      /* atomic shells */
   int nSh;                           /* # of shells */
   struct STRXSHELL *psSh;            /* [nShell] */
};

                                      /* structure to describe target material */
struct STRXTARG {
   struct Node sLN; 
   unsigned long ulFlag;              /* holds flag bits */
   unsigned long ulSpace;
   unsigned long ulCode;              /* material code number */ 
/*   char *pcMaterial; */                  /* material name */
   char *pcComposition;               /* chemical composition */
   double dDensity, dMolWeight;       /* density [g/cm**3], molweight [g] */
   double dnMolecule, dnNuclei;       /* molecule, nuclei number density */
                                      /* NA / MolWeight */
   int nAtom;                         /* # of different atoms */ 
   struct STRXATOM *psAtom;           /* [nAtom] */
                                      /* molecular shells */  
   int nSh;                           /* # of shells */
   struct STRXSHELL *psSh;            /* [nSh] */
   struct SCMDERR sCE;
};
                                      /* commonly used short codes */
enum {
   TRXTARG_NONE=0, 
   TRXTARG_H2O=1, TRXTARG_C, TRXTARG_C2H4, TRXTARG_N2, TRXTARG_O2,
   TRXTARG_PMMA,
   TRXTARG_AL, TRXTARG_NAI, TRXTARG_AGBR, 
   TRXTARG_MYLAR,
   TRXTARG_TEA,
};				      
                                      /* flags */
#define TRXTARGFILEVERSION 19990513

#define TRXTARG_REFCOPY 0x00000001    /* do not free() */ 

/*

   Functions to handle a single target
   -----------------------------------

   TRXTargAtomsFromComposition: build atom list from formula
                          14 x to test
			       o atomic shell levels
			       o kinetic energies
                             o calls:
			       o TRXProjInit()
			       o TRXProjFree()
			       o TRXProjFromStr()
                             
   TRXTargComplete         : complete target structure
                          15 x to test 
                             o calls
                               o TRXTargAtomsFromComposition()
			       o TRXTargMerge()
			        
   TRXTargCopy             : reference copy
                             o todo
			     o calls:
			      
   TRXTargDup              : duplicate target structure
                           8 x to test
                             o calls:
		               o TRXTargInit
                               o TRXProjInit
			        
   TRXTargFree             : duplicate target structure
                           3 x to test
                             o calls:
		               o TRXTargInit
                     
   TRXTargImportComposition:  
                           4 x to test
                             o calls:
                     
   TRXTargImportMaterial   :  
                           5 x to test
                             o calls:
                     
   TRXTargInit             : initialize target structure
                           2 x to test
                             o calls:
                   
   TRXTargInspect          : inspect output
                           1 x to test
                             o calls:
                   
   TRXTargIsValid          : check validity
                           6 x to test
                             ? check valid shells too 
                             o calls:
                   
   TRXTargList             : 
                          11 x to test
                             o should insert '*' before each line 
                               or separate call to _prtcl()
                             o calls:
                               o TRXTarg2Str

   TRXTargMatch            : ( to, fr, flag )
                           9 x to test
                             ? check complete atom/shell match
                             o calls
                               o TRXTargIsValid
			                 
   TRXTargMerge            : ( to, fr, flag )
                           7 x to test
                             ? merge material name too
                             o calls
                             
   TRXTargRead             : 
                          16 x to test
                             o composition
                             o density/molweight/shells optional
                             o calls:
                               o TRXTargComplete
                               o TRXTargInit                   
                               o TRXTargImportMaterial
			       o TRXTargImportComposition
			       o TRXTargFree
			         
   TRXTargSpace            : evaluate space occupied
                          13 x to test
                             ! space count too low ? 454 vs 482 expected
                             o calls:
                                
   TRXTargWrite            : 
                          12 x to test
                             o calls:
                               o TRXTarg2Str
                   
   TRXTarg2Str             :
                          10 x to test
                             o calls:
                                   
  _TRXTargTest             : special: returns ptr to H2O target structure
                             o to test
		             o calls:		     


   Functions to handle target base
   -------------------------------

   TRXTargBaseDelete  : 
                      9 o to test
                        o calls:
                          o TRXTargFree
   TRXTargBaseFree    : 
                      3 o to test
                        o calls:
                          o TRXTargFree
                          o TRXBaseInit
   TRXTargBaseInit    : 
                      2 o to test
                        o calls:
   TRXTargBaseInspect : 
                      1 o to test
                        o calls:
                          o TRXTargInspect
   TRXTargBaseIsValid : 
                      4 o to test
                        o calls:
                          o TRXTargIsValid
   TRXTargBaseList    : 
                      5 o to test
                        o calls:
                          o TRXTargList
   TRXTargBaseLocate  : 
                      6 o to test
                        o calls:
   TRXTargBaseRead    : 
                      7 o to test
                        o calls:
                          o TRXTargRead
                          o TRXTargFree
                          o TRXTargBaseLocate
   TRXTargBaseWrite   : 
                      8 o to test
                        o calls:
                          o TRXTargWrite  

   

   TRXTargError  : error handling

*/
                                      /* function prototypes */
int TRXTargAtomsEKinetic( struct STRXTARG *, unsigned long );

int TRXTargComplete( struct STRXTARG *, unsigned long );
int TRXTargDup     ( struct STRXTARG *, struct STRXTARG *, unsigned long );
int TRXTargFree    ( struct STRXTARG *, unsigned long );

int TRXTargImportComposition( struct STRXTARG *, const char *, unsigned long );
int TRXTargImportMaterial   ( struct STRXTARG *, const char *, unsigned long );

int TRXTargInit    ( struct STRXTARG *, unsigned long );
int TRXTargInspect ( struct STRXTARG *, unsigned long );
int TRXTargIsValid ( struct STRXTARG *, unsigned long );
int TRXTargList    ( struct STRXTARG *, unsigned long );
int TRXTargMatch   ( struct STRXTARG *, struct STRXTARG *, unsigned long );
int TRXTargMerge   ( struct STRXTARG *, struct STRXTARG *, unsigned long );
int TRXTargRead    ( struct STRXTARG *, const char *, unsigned long );
int TRXTargSpace   ( struct STRXTARG *, unsigned long );
int TRXTargWrite   ( struct STRXTARG *, const char *, unsigned long );
int TRXTarg2Str    ( struct STRXTARG *, char *, unsigned long );

struct STRXTARG *_TRXTargTest ( unsigned long );
struct STRXTARG *_TRXTargTest2( unsigned long );



                                      /* all defined materials */
struct STRXTARGBASE {
   struct List sLH;
   struct SCMDERR sCE;
};

int TRXTargBaseDelete ( struct STRXTARGBASE *, const char *, unsigned long );
int TRXTargBaseFree   ( struct STRXTARGBASE *, unsigned long );
int TRXTargBaseInit   ( struct STRXTARGBASE *, unsigned long );
int TRXTargBaseInspect( struct STRXTARGBASE *, const char *, unsigned long );
int TRXTargBaseIsValid( struct STRXTARGBASE *, unsigned long );
int TRXTargBaseList   ( struct STRXTARGBASE *, const char *, unsigned long );
int TRXTargBaseRead   ( struct STRXTARGBASE *, const char *, unsigned long );
int TRXTargBaseWrite  ( struct STRXTARGBASE *, const char *, const char *, unsigned long );

int TRXTargError( struct SCMDERR *, unsigned long );

/*
   --- error codes
*/
enum { ERRTRXTARG_SYS=1, 
       ERRTRXTARG_BADADDR,
       ERRTRXTARG_MISMATCHNAME,
       ERRTRXTARG_BADNAME,
       ERRTRXTARG_BADDENSITY,
       ERRTRXTARG_BADMOLWEIGHT,
       ERRTRXTARG_BADATOM,
       ERRTRXTARG_NOSUCHTARGET,
       ERRTRXTARG_NOFILE,
       ERRTRXTARG_CMDPARNOTYET,
       ERRTRXTARG_BADMATERIAL,
       ERRTRXTARG_NOSUCHMATERIAL,
       ERRTRXTARG_NOMOLWEIGHT,
       ERRTRXTARG_BADFILETYPE,
       ERRTRXTARG_BADFILEVERS,
       ERRTRXTARG_ITEMNOTYET,
       ERRTRXTARG_UNKNOWNTAG,
       ERRTRXTARG_SYNTAX,
       ERRTRXTARG_BADSHELL,
       ERRTRXTARG_BADFILELIST,
       ERRTRXTARG_BASEEMPTY,
       ERRTRXTARG_NAMEMISMATCH,
       ERRTRXTARG_COMPMISMATCH,  
       ERRTRXTARG_DENSMISMATCH,  
       ERRTRXTARG_MOLWMISMATCH,  
       ERRTRXTARG_NMOLMISMATCH,  
       ERRTRXTARG_NNUCMISMATCH,  
       ERRTRXTARG_SHELLMISMATCH,  
       ERRTRXTARG_ATOMMISMATCH,  
       ERRTRXTARG_BADCOMPOSITION,
       ERRTRXTARG_BADELEMENT,
       ERRTRXTARG_NOSUCHATOM,
       ERRTRXTARG_NOSUCHSHELL,
       ERRTRXTARG_BADNUMBER,
       ERRTRXTARG_BADPARENT,
};

#endif
